import { reactive } from 'vue'

import * as env from '@/lib/env.js'
import { checkCanvas, checkMedia } from '@/lib/checks.js'

const store = reactive({
  scores: 80,
  useragent: navigator.userAgent,
  env,

  modules: [
    {
      name: '音频支持',
      total: 0,
      factor: 80,
      scores: [
        !!window.AudioContext ^ 0,
        checkMedia('audio/wav'),
        checkMedia('audio/mpeg'),
        checkMedia('audio/flac'),
        checkMedia('audio/aac'),
        checkMedia('audio/ogg'),
        checkMedia('audio/webm')
      ],
      items: [
        'Web Audio API',
        'pcm audio support',
        'mp3 support',
        'flac support',
        'aac support',
        'ogg support',
        'webm support',
        'Dolby Digital support',
        'Dolby Digital Plus support'
      ]
    },
    {
      name: '视频支持',
      total: 0,
      factor: 80,
      scores: [
        checkMedia('video/mp4; codecs="mp4v.20.8"', 0),
        checkMedia('video/mp4', 0),
        checkMedia('video/mp4; codecs="hev1"', 0),
        checkMedia('video/ogg', 0),
        checkMedia('video/webm; codecs="vp8"', 0),
        checkMedia('video/webm; codecs="vp9"', 0)
      ],
      items: [
        'MPEG-4 ASP support',
        'H.264 support',
        'H.265 support',
        'Ogg Theora support',
        'WebM with VP8 support',
        'WebM with VP9 support'
      ]
    },
    {
      name: '流媒体',
      total: 0,
      scores: [1, 1, 0, 0],
      factor: 100,
      items: [
        'Media Source extensions',
        'DRM support',
        'Dynamic Adaptive Streaming / MPEG-DASH',
        'HTTP Live Streaming / HLS'
      ]
    },
    {
      name: '自定义组件',
      total: 0,
      factor: 80,
      scores: [
        !!window.customElements ^ 0,
        !!document.body.attachShadow ^ 0,
        !!document.adoptedStyleSheets ^ 0
      ],
      items: ['Custom elements', 'Shadow DOM', 'adoptedStyleSheets']
    },
    {
      name: '图形图象支持',
      total: 0,
      factor: 120,
      scores: [
        checkCanvas('2d'),
        checkCanvas('webgl'),
        checkCanvas('webgl2'),
        checkCanvas('webgpu'),
        !!navigator.xr ^ 0
      ],
      items: ['canvas 2D', 'WebGL', 'WebGL 2', 'WebGPU', 'WebVR']
    },
    {
      name: '网络',
      total: 0,
      factor: 60,
      scores: [
        !!window.EventSource ^ 0,
        !!window.WebSocket ^ 0,
        !!window.fetch ^ 0,
        !!File.prototype.arrayBuffer ^ 0
      ],
      items: [
        'Server-Sent events',
        'websocket',
        'fetch api',
        'ArrayBuffer and Blob support'
      ]
    }
  ]
})

store.modules.forEach(it => (it.total = it.items.length))
store.scores = store.modules.map(it => it.scores.sum() * it.factor).sum()

export default function (app) {
  app.config.globalProperties.$store = store
}
